@@ -100,7 +100,7 @@ ROM_KANA_RULE = {
     # u'h': (u'', (u'オ', u'お')),
     u'ha': (None, (u'ハ', u'は')),
     u'he': (None, (u'ヘ', u'へ')),
-    u'hh': (u'h', (u'ッ', u'っ')),
+    u'hh': (u'h', (u'ッ', u'っ')), # skk-rom-kana-list
     u'hi': (None, (u'ヒ', u'ひ')),
     u'ho': (None, (u'ホ', u'ほ')),
     u'hu': (None, (u'フ', u'ふ')),
@@ -135,6 +135,7 @@ ROM_KANA_RULE = {
     u'ma': (None, (u'マ', u'ま')),
     u'me': (None, (u'メ', u'め')),
     u'mi': (None, (u'ミ', u'み')),
+    u'mm': (u'm', (u'ッ', u'っ')), # skk-rom-kana-list
     u'mo': (None, (u'モ', u'も')),
     u'mu': (None, (u'ム', u'む')),
     u'mya': (None, (u'ミャ', u'みゃ')),
@@ -837,6 +838,10 @@ class SkkServ(DictBase):
             response = str()
             while '\n' not in response:
                 response += self.__socket.recv(self.BUFSIZ)
+                # skksearch does not terminate the line with LF on
+                # error (Issue#30)
+                if len(response) > 0 and response[0] != '1':
+                    break
             if len(response) == 0 or response[0] != '1':
                 return list()
             return self.split_candidates(response.decode(self.__encoding)[1:])
@@ -851,6 +856,10 @@ class SkkServ(DictBase):
             response = str()
             while '\n' not in response:
                 response += self.__socket.recv(self.BUFSIZ)
+                # skksearch does not terminate the line with LF on
+                # error (Issue#30)
+                if len(response) > 0 and response[0] != '1':
+                    break
             if len(response) < 2 or response[0] != '1':
                 return iter(list())
             return iter(response[2:-1].decode(self.__encoding).split(response[1]))
@@ -1294,8 +1303,14 @@ class Context(object):
         if key.is_nicola():
             return False
         output, pending, tree = self.__current_state().rom_kana_state
-        return len(pending) > 0 and \
-            key.letter.lower() in self.__current_state().rom_kana_state[2]
+        if len(pending) == 0:
+            return False
+        if key.letter.lower() not in tree:
+            return False
+        arg = tree[key.letter.lower()]
+        if isinstance(arg, tuple) and arg[0]: # arg[0] == next_pending
+            return False
+        return True
 
     def __get_next_input_mode(self, key):
         input_mode = INPUT_MODE_TRANSITION_RULE.get(str(key), dict()).\
@@ -1609,6 +1624,8 @@ class Context(object):
                     not self.__rom_kana_key_is_acceptable(key)) or \
                     (key.is_nicola() and key.letter == '[fj]'):
                 rom_kana_state = self.__convert_nn(self.__current_state().rom_kana_state)
+                rom_kana_state = self.__finish_next_pending(key.letter.lower(),
+                                                            rom_kana_state)
                 if len(rom_kana_state[1]) == 0 and \
                         not self.__current_state().okuri_rom_kana_state:
                     self.__current_state().rom_kana_state = rom_kana_state
@@ -1905,13 +1922,34 @@ elements will be "[[DictEdit]] かんが*え ", "▽", "かんが", "*え" .'''
                 output += u'ﾝ'
             return (output, pending[:-1], self.__rom_kana_rule_tree)
         return state
-        
+
+    def __convert_next_output(self, next_output, state):
+        output, pending, tree = state
+        if isinstance(next_output, unicode):
+            output += next_output
+        else:
+            output += self.__convert_kana_by_input_mode(*next_output)
+        return (output, u'', self.__rom_kana_rule_tree)
+
+    def __convert_next_pending(self, next_pending, state):
+        for next_letter in next_pending:
+            state = self.__convert_rom_kana(next_letter, state)
+        return state
+
+    def __finish_next_pending(self, letter, state):
+        output, pending, tree = state
+        if letter in tree and isinstance(tree[letter], tuple) and \
+                tree[letter][0] is not None:
+            output, pending, tree = self.__convert_rom_kana(letter, state)
+            return (output, u'', self.__rom_kana_rule_tree)
+        return state
+
     def __convert_kana(self, key, state):
         if key.is_nicola():
             return self.__convert_nicola_kana(key, state)
         else:
             return self.__convert_rom_kana(key.letter.lower(), state)
-            
+
     def __convert_rom_kana(self, letter, state):
         output, pending, tree = state
         if letter not in tree:
@@ -1928,16 +1966,10 @@ elements will be "[[DictEdit]] かんが*え ", "▽", "かんが", "*え" .'''
         if isinstance(tree[letter], dict):
             return (output, pending + letter, tree[letter])
         next_pending, next_output = tree[letter]
-        if isinstance(next_output, unicode):
-            output += next_output
-        else:
-            katakana, hiragana = next_output
-            output += self.__convert_kana_by_input_mode(katakana, hiragana)
-        next_state = (output, u'', self.__rom_kana_rule_tree)
+        state = self.__convert_next_output(next_output, state)
         if next_pending:
-            for next_letter in next_pending:
-                next_state = self.__convert_rom_kana(next_letter, next_state)
-        return next_state
+            state = self.__convert_next_pending(next_pending, state)
+        return state
 
     def __convert_nicola_kana(self, key, state):
         assert key.is_nicola()
